@@ -48,10 +48,10 @@ int on_client_hello_h3_cb(ptls_on_client_hello_t *self, ptls_t *ptls,
 
   for (size_t i = 0; i < negprotos.count; ++i) {
     auto &proto = negprotos.list[i];
-    if (H3_ALPN_V1[0] == proto.len &&
-        memcmp(&H3_ALPN_V1[1], proto.base, proto.len) == 0) {
+    if (std::ranges::equal(H3_ALPN_V1.subspan(1),
+                           std::span{proto.base, proto.len})) {
       if (ptls_set_negotiated_protocol(
-              ptls, reinterpret_cast<char *>(proto.base), proto.len) != 0) {
+            ptls, reinterpret_cast<char *>(proto.base), proto.len) != 0) {
         return -1;
       }
 
@@ -72,10 +72,10 @@ int on_client_hello_hq_cb(ptls_on_client_hello_t *self, ptls_t *ptls,
 
   for (size_t i = 0; i < negprotos.count; ++i) {
     auto &proto = negprotos.list[i];
-    if (HQ_ALPN_V1[0] == proto.len &&
-        memcmp(&HQ_ALPN_V1[1], proto.base, proto.len) == 0) {
+    if (std::ranges::equal(HQ_ALPN_V1.subspan(1),
+                           std::span{proto.base, proto.len})) {
       if (ptls_set_negotiated_protocol(
-              ptls, reinterpret_cast<char *>(proto.base), proto.len) != 0) {
+            ptls, reinterpret_cast<char *>(proto.base), proto.len) != 0) {
         return -1;
       }
 
@@ -92,25 +92,21 @@ ptls_on_client_hello_t on_client_hello_hq = {on_client_hello_hq_cb};
 namespace {
 auto ticket_hmac = EVP_sha256();
 
-template <size_t N> void random_bytes(std::array<uint8_t, N> &dest) {
-  ptls_openssl_random_bytes(dest.data(), dest.size());
-}
-
-const std::array<uint8_t, 16> &get_ticket_key_name() {
+std::span<const uint8_t> get_ticket_key_name() {
   static std::array<uint8_t, 16> key_name;
-  random_bytes(key_name);
+  ptls_openssl_random_bytes(key_name.data(), key_name.size());
   return key_name;
 }
 
-const std::array<uint8_t, 32> &get_ticket_key() {
+std::span<const uint8_t> get_ticket_key() {
   static std::array<uint8_t, 32> key;
-  random_bytes(key);
+  ptls_openssl_random_bytes(key.data(), key.size());
   return key;
 }
 
-const std::array<uint8_t, 32> &get_ticket_hmac_key() {
+std::span<const uint8_t> get_ticket_hmac_key() {
   static std::array<uint8_t, 32> hmac_key;
-  random_bytes(hmac_key);
+  ptls_openssl_random_bytes(hmac_key.data(), hmac_key.size());
   return hmac_key;
 }
 } // namespace
@@ -124,14 +120,14 @@ int ticket_key_cb(unsigned char *key_name, unsigned char *iv,
                   HMAC_CTX *hctx,
 #endif // OPENSSL_VERSION_NUMBER < 0x30000000L
                   int enc) {
-  static const auto &static_key_name = get_ticket_key_name();
-  static const auto &static_key = get_ticket_key();
-  static const auto &static_hmac_key = get_ticket_hmac_key();
+  static const auto static_key_name = get_ticket_key_name();
+  static const auto static_key = get_ticket_key();
+  static const auto static_hmac_key = get_ticket_hmac_key();
 
   if (enc) {
     ptls_openssl_random_bytes(iv, EVP_MAX_IV_LENGTH);
 
-    memcpy(key_name, static_key_name.data(), static_key_name.size());
+    std::ranges::copy(static_key_name, key_name);
 
     if (!EVP_EncryptInit_ex(ctx, EVP_aes_256_cbc(), nullptr, static_key.data(),
                             iv)) {
@@ -139,13 +135,13 @@ int ticket_key_cb(unsigned char *key_name, unsigned char *iv,
     }
 #if OPENSSL_VERSION_NUMBER >= 0x30000000L
     auto params = std::to_array({
-        OSSL_PARAM_construct_octet_string(
-            OSSL_MAC_PARAM_KEY, const_cast<uint8_t *>(static_hmac_key.data()),
-            static_hmac_key.size()),
-        OSSL_PARAM_construct_utf8_string(
-            OSSL_MAC_PARAM_DIGEST,
-            const_cast<char *>(EVP_MD_get0_name(ticket_hmac)), 0),
-        OSSL_PARAM_construct_end(),
+      OSSL_PARAM_construct_octet_string(
+        OSSL_MAC_PARAM_KEY, const_cast<uint8_t *>(static_hmac_key.data()),
+        static_hmac_key.size()),
+      OSSL_PARAM_construct_utf8_string(
+        OSSL_MAC_PARAM_DIGEST,
+        const_cast<char *>(EVP_MD_get0_name(ticket_hmac)), 0),
+      OSSL_PARAM_construct_end(),
     });
     if (!EVP_MAC_CTX_set_params(hctx, params.data())) {
       /* TODO Which value should we return on error? */
@@ -161,7 +157,8 @@ int ticket_key_cb(unsigned char *key_name, unsigned char *iv,
     return 1;
   }
 
-  if (memcmp(key_name, static_key_name.data(), static_key_name.size()) != 0) {
+  if (!std::ranges::equal(std::span{key_name, static_key_name.size()},
+                          static_key_name)) {
     return 0;
   }
 
@@ -171,13 +168,13 @@ int ticket_key_cb(unsigned char *key_name, unsigned char *iv,
   }
 #if OPENSSL_VERSION_NUMBER >= 0x30000000L
   auto params = std::to_array({
-      OSSL_PARAM_construct_octet_string(
-          OSSL_MAC_PARAM_KEY, const_cast<uint8_t *>(static_hmac_key.data()),
-          static_hmac_key.size()),
-      OSSL_PARAM_construct_utf8_string(
-          OSSL_MAC_PARAM_DIGEST,
-          const_cast<char *>(EVP_MD_get0_name(ticket_hmac)), 0),
-      OSSL_PARAM_construct_end(),
+    OSSL_PARAM_construct_octet_string(
+      OSSL_MAC_PARAM_KEY, const_cast<uint8_t *>(static_hmac_key.data()),
+      static_hmac_key.size()),
+    OSSL_PARAM_construct_utf8_string(
+      OSSL_MAC_PARAM_DIGEST, const_cast<char *>(EVP_MD_get0_name(ticket_hmac)),
+      0),
+    OSSL_PARAM_construct_end(),
   });
   if (!EVP_MAC_CTX_set_params(hctx, params.data())) {
     /* TODO Which value should we return on error? */
@@ -199,7 +196,7 @@ int encrypt_ticket_cb(ptls_encrypt_ticket_t *encrypt_ticket, ptls_t *ptls,
                       int is_encrypt, ptls_buffer_t *dst, ptls_iovec_t src) {
   int rv;
   auto conn_ref =
-      static_cast<ngtcp2_crypto_conn_ref *>(*ptls_get_data_ptr(ptls));
+    static_cast<ngtcp2_crypto_conn_ref *>(*ptls_get_data_ptr(ptls));
   auto conn = conn_ref->get_conn(conn_ref);
   uint32_t ver;
 
@@ -208,8 +205,9 @@ int encrypt_ticket_cb(ptls_encrypt_ticket_t *encrypt_ticket, ptls_t *ptls,
     // TODO Replace std::make_unique with
     // std::make_unique_for_overwrite when it is available.
     auto buf = std::make_unique<uint8_t[]>(src.len + sizeof(ver));
-    auto p = std::copy_n(src.base, src.len, buf.get());
-    p = std::copy_n(reinterpret_cast<uint8_t *>(&ver), sizeof(ver), p);
+    auto p = std::ranges::copy_n(src.base, src.len, buf.get()).out;
+    p = std::ranges::copy_n(reinterpret_cast<uint8_t *>(&ver), sizeof(ver), p)
+          .out;
 
     src.base = buf.get();
     src.len = p - buf.get();
@@ -255,20 +253,22 @@ ptls_encrypt_ticket_t encrypt_ticket = {encrypt_ticket_cb};
 
 namespace {
 ptls_key_exchange_algorithm_t *key_exchanges[] = {
-    &ptls_openssl_x25519,
-    &ptls_openssl_secp256r1,
-    &ptls_openssl_secp384r1,
-    &ptls_openssl_secp521r1,
-    nullptr,
+#if PTLS_OPENSSL_HAVE_X25519
+  &ptls_openssl_x25519,
+#endif // PTLS_OPENSSL_X25519
+  &ptls_openssl_secp256r1, &ptls_openssl_secp384r1,
+  &ptls_openssl_secp521r1, nullptr,
 };
 } // namespace
 
 namespace {
 ptls_cipher_suite_t *cipher_suites[] = {
-    &ptls_openssl_aes128gcmsha256,
-    &ptls_openssl_aes256gcmsha384,
-    &ptls_openssl_chacha20poly1305sha256,
-    nullptr,
+  &ptls_openssl_aes128gcmsha256,
+  &ptls_openssl_aes256gcmsha384,
+#if PTLS_OPENSSL_HAVE_CHACHA20_POLY1305
+  &ptls_openssl_chacha20poly1305sha256,
+#endif // PTLS_OPENSSL_CHACHA20POLY1305SHA256
+  nullptr,
 };
 } // namespace
 
